#!/usr/bin/env node
"use strict";

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path2 = require("path");

var _path3 = _interopRequireDefault(_path2);

var _fs = require("fs");

var _yargs = require("yargs");

var _yargs2 = _interopRequireDefault(_yargs);

var _updateNotifier = require("update-notifier");

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _config = require("./config");

var _config2 = _interopRequireDefault(_config);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

var _utils = require("./utils");

var _combiner2 = require("./combiner");

var _combiner3 = _interopRequireDefault(_combiner2);

var _help = require("./lib/help");

var _help2 = _interopRequireDefault(_help);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(function (parser) {
	// wrap in IIFE to be able to use return
	var yargs = parser.usage(_help2.default);

	require("./config-yargs")(yargs);

	yargs.command(["save", "$0"], "Find diffs between your findBy property value with current messages in the application.").help();

	yargs.command(["merge", "$0"], "Merge diffs with your locale file which you have put it by findBy property via by terminal or combinerrc file.").help();

	yargs.command(["delete", "$0"], "delete the created diff file").help();

	// yargs will terminate the process early when the user uses help or version.
	// This causes large help outputs to be cut short (https://github.com/nodejs/node/wiki/API-changes-between-v0.10-and-v4#process).
	// To prevent this we use the yargs.parse API and exit the process normally
	yargs.parse(process.argv.slice(2), function (err, argv, output) {
		Error.stackTraceLimit = 30;
		var terminalConfig = {
			locales: [],
			findBy: "",
			context: "",
			diffPath: "",
			messages: ""
		};

		// arguments validation failed
		if (err && output) {
			console.error(output);
			process.exitCode = 1;
			return;
		}

		// help or version info
		if (output) {
			console.log(output);
			return;
		}

		var ifArg = (0, _utils.ifArg)(argv);

		ifArg("locale", function (locale) {
			if (locale.includes(":")) {
				var _locale$split = locale.split(":"),
				    _locale$split2 = _slicedToArray(_locale$split, 2),
				    name = _locale$split2[0],
				    _path = _locale$split2[1];

				terminalConfig.locales.push({ name: name, path: _path });
			}
		});

		ifArg("context", function (context) {
			if (context) {
				terminalConfig.context = context;
			}
		});

		ifArg("diff-path", function (diffPath) {
			if (diffPath) {
				terminalConfig.diffPath = diffPath;
			}
		});

		ifArg("messages", function (messages) {
			if (messages) {
				terminalConfig.messages = messages;
			}
		});

		ifArg("findBy", function (findBy) {
			if (findBy) {
				terminalConfig.findBy = findBy;
			}
		});

		function completeConfig(config) {
			function join(lastPath) {
				// Global system address by using the process.cwd() native function
				return _path3.default.join(process.cwd(), config.context, lastPath);
			}
			if (config.context) {
				config.locales = _ramda2.default.map(function (locale) {
					locale.path = join(locale.path);

					return locale;
				})(config.locales);
				config.messages = join(config.messages);
				if (config.diffPath) {
					config.diffPath = join(config.diffPath);
				} else {
					config.diffPath = _path3.default.dirname(config.locales[0].path) + "/diff.json";
				}
			} else {
				// No context found
				if (!config.diffPath && !_ramda2.default.isEmpty(config.locales)) {
					config.diffPath = _path3.default.dirname(config.locales[0].path) + "/diff.json";
				}
			}

			return config;
		}

		var _argv$_ = _slicedToArray(argv._, 1),
		    eventType = _argv$_[0];

		if ((0, _utils.validation)(terminalConfig)) {
			terminalConfig = completeConfig(terminalConfig);

			if ((0, _utils.validation)(terminalConfig)) {
				if (showNotFoundErrors(terminalConfig.locales)) return false;

				if ((0, _utils.existLocale)(terminalConfig.findBy, terminalConfig.locales)) {
					var _terminalConfig = terminalConfig,
					    findBy = _terminalConfig.findBy,
					    locales = _terminalConfig.locales,
					    diffPath = _terminalConfig.diffPath,
					    messages = _terminalConfig.messages;

					var locale = _ramda2.default.find(_ramda2.default.propEq("name", findBy))(locales).path;

					var combiner = new _combiner3.default({
						messages: messages,
						diffPath: diffPath,
						locale: locale
					});

					if (eventType === "save") {
						combiner.saveMessages().getDiff().saveDiff();
					} else if (eventType === "merge") {
						combiner.loadDiffs().loadLocale().mergeDiff();
					} else if (eventType === "delete") {
						deleteFile(diffPath);
					}
				}
			} else {
				(0, _utils.displayError)((0, _utils.saveErrors)(terminalConfig));
			}
		} else {
			var localConfig = _extends({}, _config2.default, _ramda2.default.filter(function (value) {
				return !_ramda2.default.isEmpty(value);
			})(terminalConfig));
			localConfig = completeConfig(localConfig);
			// console.log("localConfig", localConfig);

			if ((0, _utils.validation)(localConfig)) {
				if (showNotFoundErrors(localConfig.locales)) return false;

				if ((0, _utils.existLocale)(localConfig.findBy, localConfig.locales)) {
					var _localConfig = localConfig,
					    _messages = _localConfig.messages,
					    _diffPath = _localConfig.diffPath,
					    _locales = _localConfig.locales,
					    _findBy = _localConfig.findBy;

					var _locale = _ramda2.default.find(_ramda2.default.propEq("name", _findBy))(_locales).path;

					var _combiner = new _combiner3.default({
						messages: _messages,
						diffPath: _diffPath,
						locale: _locale
					});

					if (eventType === "save") {
						_combiner.saveMessages().getDiff().saveDiff();
					} else if (eventType === "merge") {
						_combiner.loadDiffs().loadLocale().mergeDiff();
					} else if (eventType === "delete") {
						deleteFile(_diffPath);
					}
				}
			} else {
				(0, _utils.displayError)((0, _utils.saveErrors)(localConfig));
			}
		}
	});

	function showNotFoundErrors(list) {
		var errors = (0, _utils.checkFileExists)(list);
		if (!_ramda2.default.isEmpty((0, _utils.checkFileExists)(list))) {
			console.log(_chalk2.default.yellow("These following files don't exist, make sure you have written the right paths:"), "\n");
			console.log(_chalk2.default.red(errors.join("\n")));
			return true;
		}

		return false;
	}

	function deleteFile(file) {
		(0, _fs.unlink)(file, function (err) {
			if (err) {
				console.log(_chalk2.default.white.bgRed.bold(err.message));

				return;
			}
			console.log(_chalk2.default.white.bgGreen.bold("successfully deleted ", (0, _utils.relativePath)(file)));
		});
	}

	(0, _updateNotifier2.default)({ pkg: _package2.default }).notify();
})(_yargs2.default);