"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

// Ramda functions


var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _glob = require("glob");

var _glob2 = _interopRequireDefault(_glob);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _babelCore = require("babel-core");

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var IntlCombiner = function () {
	function IntlCombiner(_ref) {
		var _ref$diffPath = _ref.diffPath,
		    diffPath = _ref$diffPath === undefined ? (0, _utils.required)("diffPath") : _ref$diffPath,
		    _ref$locale = _ref.locale,
		    locale = _ref$locale === undefined ? (0, _utils.required)("locale") : _ref$locale,
		    _ref$messages = _ref.messages,
		    messages = _ref$messages === undefined ? (0, _utils.required)("messages") : _ref$messages;

		_classCallCheck(this, IntlCombiner);

		this.diffFile = diffPath;
		this.localeFile = locale;
		this.messagesFile = messages;

		this.findedMessages = {};
		this.localeMessages = {};
		this.diffMessages = {};
	}

	/**
  * saveMessages - Get and find Intl messages objects
  */


	_createClass(IntlCombiner, [{
		key: "saveMessages",
		value: function saveMessages(callback) {
			var srcPaths = _glob2.default.sync(this.messagesFile, { absolute: true });
			var contents = _ramda2.default.map(function (p) {
				return _fs2.default.readFileSync(p, "utf-8");
			}, srcPaths);
			var messages = _ramda2.default.map(function (content) {
				return (0, _babelCore.transform)(content, {
					presets: [require.resolve("babel-preset-react-app")],
					plugins: [require.resolve("babel-plugin-react-intl")],
					babelrc: false
				});
			}, contents).map(_ramda2.default.path(["metadata", "react-intl", "messages"])).reduce(_ramda2.default.concat, []);

			// Make an object of the nessages array values
			var result = _ramda2.default.map(function (_ref2) {
				var id = _ref2.id,
				    defaultMessage = _ref2.defaultMessage;
				return _defineProperty({}, id, defaultMessage);
			}, messages).reduce(_ramda2.default.merge, {});

			// Assign result in findedMessages varialble.
			this.findedMessages = result;
			callback && callback(null, this.findedMessages);

			return this;
		}
		/**
   * [getDiff description]
   * @param  {Function} callback [description]
   * @return {[type]}            [description]
   */

	}, {
		key: "getDiff",
		value: function getDiff(callback) {
			// Reading the old messages in locales folder
			var localeMessagesData = _fs2.default.readFileSync(this.localeFile, "utf8");
			// Assignment to local array of previous messages
			this.localeMessages = (0, _utils.toJSON)(localeMessagesData);
			// find diffrence of previous messages with all messages
			this.diffMessages = (0, _utils.findDifference)(this.localeMessages, this.findedMessages);

			callback && callback(null, this.diffMessages);

			return this;
		}

		/**
   * Load Diff file content then save it to diffMessages variable.
   * @return {Object} This object
   */

	}, {
		key: "loadDiffs",
		value: function loadDiffs() {
			var _this = this;

			this.loadFile(this.diffFile, function (errors, diffMessages) {
				_this.diffMessages = diffMessages;
			});

			return this;
		}
		/**
   * Load locale file content then save it to locale and finded messages variable.
   * @return {Object} This object
   */

	}, {
		key: "loadLocale",
		value: function loadLocale() {
			var _this2 = this;

			this.loadFile(this.localeFile, function (errors, messages) {
				_this2.localeMessages = messages;
				_this2.findedMessages = messages;
			});

			return this;
		}
		/**
   * Load file
   * @param  {String}   filePath Path to load your file.
   * @param  {Function} callback
   * @return {Object}            This object
   */

	}, {
		key: "loadFile",
		value: function loadFile(filePath, callback) {
			// Load the diff messages file content
			var loadedFile = _fs2.default.readFileSync(filePath, "utf8");
			loadedFile = (0, _utils.toJSON)(loadedFile);

			callback && callback(null, loadedFile);

			return this;
		}

		/**
   * saveDiff - Save Diff messages as a file.
   * @return {Object} Beautified json string.
   */

	}, {
		key: "saveDiff",
		value: function saveDiff(file) {
			var diffFile = file || this.diffFile;
			var diffLength = (0, _utils.objectLength)(this.diffMessages);
			// Save diff(s) in file
			if (diffLength !== 0) {
				this.saveFile("saved", diffFile, this.diffMessages);

				console.log("\n", _chalk2.default.white.bgGreen("Number of diffs:"), _chalk2.default.blue((0, _utils.objectLength)(this.diffMessages)));
			} else {
				console.log(_chalk2.default.white.bgRed.bold("Warning:"), _chalk2.default.yellow("There is not any diffrences to find and saving"));
			}
			return this;
		}
		/**
   * mergeDiff - Merge founded diff's messages with original messages file.
   * @return {Object} this of class
   */

	}, {
		key: "mergeDiff",
		value: function mergeDiff(callback) {
			if ((0, _utils.objectLength)(this.diffMessages) > 0) {
				var assignObjects = _extends({}, this.findedMessages, this.diffMessages);
				// Save new assigned messages file
				this.saveFile("merged", this.localeFile, assignObjects);

				console.log("\n", _chalk2.default.white.bgGreen("Number of diffs:"), _chalk2.default.blue((0, _utils.objectLength)(this.diffMessages)));
			} else {
				console.log(_chalk2.default.yellow("- Warning: The original messages file has not any diffrence with all messages file for merging!"));
			}

			return this;
		}
		/**
   * saveFile - Save messages in a file
   * @return {Object} this of class
   */

	}, {
		key: "saveFile",
		value: function saveFile(type) {
			var diffFileInput = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

			var _this3 = this;

			var messages = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.findedMessages;
			var callback = arguments[3];

			var diffFile = diffFileInput || this.diffFile;

			// Write Sync file
			_fs2.default.writeFile(diffFile, (0, _utils.beautifyMessages)(messages), function (err) {
				if (err) {
					console.log(_chalk2.default.white.bgRed.bold(err.message));

					callback && callback(err.message, null);
					return;
				}

				callback && callback(null, true);
				console.log("\n", _chalk2.default.white.bgGreen(_ramda2.default.toUpper(type)), _chalk2.default.green((0, _utils.relativePath)(_this3.messagesFile) + " -> " + (0, _utils.relativePath)(diffFile)));
			});

			return this;
		}
	}]);

	return IntlCombiner;
}();

exports.default = IntlCombiner;